# -*- coding: utf-8 -*-

import datetime

import requests

try:
    from urllib.parse import urljoin
except:
    from urllib.parse import urljoin


class BaseSCenterClient(object):
    requests = requests.Session()
    base_url = 'http://supplier.cognichain.com/'

    def __init__(self, username=None, password=None, base_url=base_url, token=None, expires=None, session=None,
                 retry_time=3):
        self.retry_time = retry_time
        self.auth_ok = False
        self.username = username
        self.password = password
        self.base_url = base_url
        self.expires = expires
        self.api_base_url = urljoin(base_url, 'api/v2/')

        if token:
            self.token = token
            self.requests.headers.update({
                'Authorization': 'JWT {}'.format(token)
            })
        else:
            self.fetch_token(username, password)

    def fetch_token(self, username, password):
        """
        请求获取token（没有token或token失效都调用这里）
        :param username:
        :param password:
        :return:
        """
        url = urljoin(self.base_url, 'api-token-auth/')
        r = self.requests.post(url, json={'username': username, 'password': password})
        self.token = r.json().get('token')
        expires = r.json().get('expires')
        if expires:
            self.expires = datetime.datetime.strptime(expires, "%Y-%m-%dT%H:%M:%S.%f")
        self.requests.headers.update({
            'Authorization': 'JWT {}'.format(self.token)
        })
        self.auth_ok = True
        return self.token

    def refresh_token(self):
        """
        此接口用于在token未设置过期时间的情况下使用，传递的token必须是有效的。
        :return:
        """
        url = urljoin(self.base_url, 'api-token-refresh/')
        r = self.requests.post(url, json={'token': self.token})
        self.token = r.json().get('token')
        self.requests.headers.update({
            'Authorization': 'JWT {}'.format(self.token)
        })
        return self.token

    def get(self, url, **kwargs):
        return self.handle_response(method='get', url=url, **kwargs)

    def post(self, url, **kwargs):
        return self.handle_response(method='post', url=url, **kwargs)

    def patch(self, url, **kwargs):
        return self.handle_response(method='patch', url=url, **kwargs)

    def handle_response(self, method, url, **kwargs):
        r = None
        if method == 'get':
            r = self.requests.get(urljoin(self.api_base_url, url), **kwargs)
        elif method == 'patch':
            r = self.requests.patch(urljoin(self.api_base_url, url), **kwargs)
        elif method == 'post':
            r = self.requests.post(urljoin(self.api_base_url, url), **kwargs)

        status_code = r.status_code
        if status_code >= 400 and status_code != 404:
            if status_code == 511:
                self.auth_ok = False
                raise ValueError('平台认证出错误，请联系管理员')

            # todo 状态码统一处理
            token_error = r.json().get("detail", None)
            if token_error:
                self.fetch_token(self.username, self.password)
                r = self.requests.post(urljoin(self.api_base_url, url), **kwargs)

        elif status_code == 404:
            raise LookupError('未找该资源')
        return r



# coding=utf-8

from .base import BaseSCenterAPI


class ProductShelvesStatus(BaseSCenterAPI):
    path = 'ec_productshelvesstatuss/'

    def create(self, data):
        """
        :param data:
        :return:
        """
        return self._post(self.path, json=data).json()

    def update(self, slug, data):
        """
        :param slug:
        :param data:
        :return:
        """
        return self._patch("{}{}/".format(self.path, slug), json=data).json()

    def get(self, slug=None):
        """
        :param slug:
        :return:
        """
        return self._get("{}{}/".format(self.path, slug)).json()

    def list(self, **kwargs):
        """
        :param slug:
        :return:
        """
        return self._get(self.path, **kwargs).json()

    def down(self, slug):
        return self._post('{}{}/down/'.format(self.path, slug)).json()



# -*- coding: utf-8 -*-

import json

from .base import BaseSCenterAPI


class Delivery(BaseSCenterAPI):
    path = 'ec_deliverys/'

    def get(self, id=None):
        """
        :param slug:
        :return:
        """
        return self._get("{}{}/".format(self.path, id)).json()

    def update(self, id, data):
        return self._patch("{}{}/".format(self.path, id), json=data).json()

    def receive(self, slug, data):
        path = self.path + slug + '/receive/'
        return self._post(path, json=data).json()

    def refuse(self, slug, data):
        path = self.path + slug + '/refuse/'
        return self._post(path, json=data).json()


# -*- coding: utf-8 -*-

from .base import BaseSCenterAPI

class Product(BaseSCenterAPI):
    path = 'ec_products/'

    def get(self, slug=None):
        """
        查询商品信息
        :param slug:
        :return:
        """
        return self._get('{}{}/'.format(self.path, slug)).json()

    def list(self):
        """
        获取商品列表
        :param slug:
        :return:
        """
        return self._get(self.path).json()



# -*- coding: utf-8 -*-

from .base import BaseSCenterAPI


class Partner(BaseSCenterAPI):
    path = 'account_partners/'

    def invite_supplier_register(self, data):
        path = self.path + 'invite_supplier_register/'
        return self._post(path, json=data).json()

    def msg(self, data):
        """
        :param data:
        {
            user_slugs:['123as231','ae563ef']
            msg: {
                'title':'消息标题',
                'description':'消息描述',
                'url':'跳转链接',
            }
        }
        :return:
        """
        path = self.path + 'msg/'
        return self._post(path, json=data).json()

    def tag(self, data):
        path = self.path + 'tag/'
        return self._post(path, json=data).json()

    def get(self, slug=None):
        """
        :param slug:
        :return:
        """
        return self._get('{}{}/'.format(self.path, slug)).json()

    def list(self, page=1, name=None):
        """
        :param page: 页码
        :param name: 供应商名称
        :return:
        """
        params = {'page': page}
        if name:
            params.update({'name': name})
        return self._get(self.path, params=params).json()

        # def search(self, q_name):
        #     """
        #     按名称模糊搜索供应商
        #     :param q_name:
        #     :return:
        #     [
        #         {
        #         "id": 98,
        #         "name": "saas商城邀请的第一个供应商",
        #         "slug": "a5b675ec",
        #         "desc": null
        #         },
        #         {
        #         "id": 97,
        #         "name": "saas对接测试",
        #         "slug": "f605181d",
        #         "desc": "saas对接测试"
        #         }
        #     ]
        #     """
        #     return self.list(params={'q': q_name}).get('results', [])
        #

    def config(self, data):
        path = self.path + 'config/'
        return self._post(path, json=data).json()
